/**
* @Author: Chenyongfeng
* @email: kunbei.chen@tuya.com
* @LastEditors: Chenyongfeng
* @file name: app_battery.c
* @Description: 
* @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
* @Company: http://www.tuya.com
* @Date: 2022-02-22 09:38:57
* @LastEditTime: 2022-02-22 09:38:57
*/

#include "app_battery.h"
#include "tal_adc.h"
#include "tal_sw_timer.h"
#include "tal_memory.h"
#include "tal_heartbeat.h"
#include "app_mfg.h"
#include <math.h>

#ifndef GET_ARRAY_LEN
#define GET_ARRAY_LEN(x)    (SIZEOF(x) / SIZEOF(x[0]))
#endif

/**
 * @note CR2032 Battery characteristic
 */
#define BATTERY_DEFAULT_TABLE \
    {2400, 0}, /*minimum voltage, percentage*/ \
    {2600, 10},\
    {2800, 20},\
    {2825, 30},\
    {2850, 40},\
    {2875, 50},\
    {2900, 60},\
    {2925, 70},\
    {2950, 80},\
    {2975, 90},\
    {3000, 100},/*maximum voltage, percentage*/ \

//battery sample event and handler
TIMER_ID etimer_battery_event_id;


typedef struct {
    BOOL_T init_flag;
    BOOL_T first_rpt_real;
    UINT16_T adc_raw_mfg;
    APP_BATTERY_INFO_T* info;
    APP_BATTERY_TABLE_T* cal_table;
    UINT8_T cal_table_len;
} APP_BATTERY_RUN_DATA_T;

STATIC APP_BATTERY_TABLE_T s_battery_table_default[] = {BATTERY_DEFAULT_TABLE};
STATIC APP_BATTERY_RUN_DATA_T s_battery_run_data = { .init_flag = FALSE, 
                                                      .first_rpt_real = TRUE, 
                                                      .info = NULL,
                                                      .cal_table = NULL};


/**
* @description: 
* @param {type} 
* @return: 
*/
STATIC UINT8_T app_battery_get_battery_level(VOID_T)
{
    INT32_T adc_raw_val = 0;
    INT32_T value=1;
    UINT8_T ch = 0;
    for (ch = 0; ch < 32; ch++)
    {
        if(s_battery_run_data.info->adc.ch_list.data & value){
            break;
        }
        value<<=1;
    }

    UINT8_T ret = tal_adc_read_single_channel(BATTERY_ADC_ID, ch, &adc_raw_val);

    TAL_PR_TRACE("ret[%d],adc_raw_val[%d] ch[%d]\r\n",  ret, adc_raw_val, ch);
    //covert data
    UINT16_T adc_volt_val = 0;
    adc_volt_val = 1210 * adc_raw_val / 4095;

    s_battery_run_data.adc_raw_mfg = adc_volt_val;

    TAL_PR_TRACE("adc_volt_val[%d], ref_vol[%d], width[%d]\r\n", adc_volt_val, s_battery_run_data.info->adc.ref_vol, pow(2, s_battery_run_data.info->adc.width));
    FLOAT_T battert_level = 100;
    for(UINT8_T i = 0; i < s_battery_run_data.cal_table_len - 1; i++) {
        if ((adc_volt_val >= s_battery_run_data.cal_table[i].voltage) && (adc_volt_val <= s_battery_run_data.cal_table[i+1].voltage)) {
            //P = (P2-P1)/(V2-V1)*Vadc + P1 - (P2-P1)/(V2-V1)*V1

            TAL_PR_TRACE("%d\r\n",i);
            battert_level = (FLOAT_T)adc_volt_val * (FLOAT_T)(s_battery_run_data.cal_table[i+1].percent - s_battery_run_data.cal_table[i].percent);
            TAL_PR_TRACE("1battert[%d]\r\n",(INT32_T)(battert_level * 10000));
            
            battert_level /= (FLOAT_T)(s_battery_run_data.cal_table[i+1].voltage - s_battery_run_data.cal_table[i].voltage);
            TAL_PR_TRACE("2battert[%d]\r\n",(INT32_T)(battert_level * 10000));
            
            battert_level += s_battery_run_data.cal_table[i].percent;
            TAL_PR_TRACE("3battert[%d]\r\n",(INT32_T)(battert_level * 10000));

            battert_level -= (FLOAT_T)s_battery_run_data.cal_table[i].voltage * (FLOAT_T)(s_battery_run_data.cal_table[i+1].percent - s_battery_run_data.cal_table[i].percent) / (FLOAT_T)(s_battery_run_data.cal_table[i+1].voltage - s_battery_run_data.cal_table[i].voltage);
            TAL_PR_TRACE("4battert[%d]\r\n",(INT32_T)(battert_level * 10000));

        }
    }

    if (adc_volt_val < s_battery_run_data.cal_table[0].voltage) {
        battert_level = 0;
    } else if (adc_volt_val > s_battery_run_data.cal_table[s_battery_run_data.cal_table_len - 1].voltage) {
        battert_level = 100;
    }


    TAL_PR_TRACE("battert_level[%d]\r\n", (INT32_T)(battert_level * 10000));

    return (UINT8_T)(battert_level * 2);
}


/**
 * @description: battery event
 * @param [in] timer_id: timer id
 * @param[in] args: null
 * @return: none
 */
STATIC VOID_T app_battery_event_handler(TIMER_ID timer_id, VOID_T *arg)
{
    UINT8_T bat_level = app_battery_get_battery_level();

    if (s_battery_run_data.first_rpt_real) {
        s_battery_run_data.first_rpt_real = FALSE;

        if (mfg_battery_test_callback(s_battery_run_data.adc_raw_mfg)) {
            return;
        }

        //sample and report
        app_battery_update_and_report_attribute(0x01, bat_level);
        tal_sw_timer_start(etimer_battery_event_id, s_battery_run_data.info->cap_max_period_time, TAL_TIMER_CYCLE);
        return;
    }

    //sample and check value, check write attribute or not.
    app_battery_update_attribute(0x01, bat_level);
}

/**
* @description: 
* @param {type} 
* @return: 
*/
VOID_T app_battery_config_init(APP_BATTERY_INFO_T* bat_info, APP_BATTERY_TABLE_T* table, UINT8_T table_sums)
{
    if (bat_info == NULL) {
        return;
    }

    s_battery_run_data.info = (APP_BATTERY_INFO_T *)tal_malloc(SIZEOF(APP_BATTERY_INFO_T));
    if (s_battery_run_data.info == NULL) {
        TAL_PR_TRACE("s_battery_run_data.info malloc error!\r\n");
        return;
    }
    tal_system_memset(s_battery_run_data.info, 0, SIZEOF(APP_BATTERY_INFO_T));
    tal_system_memcpy(s_battery_run_data.info, bat_info, SIZEOF(APP_BATTERY_INFO_T));

    if ((table != NULL) && (table_sums != 0)) {
        s_battery_run_data.cal_table = (APP_BATTERY_TABLE_T *)tal_malloc(table_sums * SIZEOF(APP_BATTERY_TABLE_T));
        if (s_battery_run_data.cal_table == NULL) {
            TAL_PR_TRACE("s_battery_run_data.cal_table malloc error!\r\n");
            return;
        }
        tal_system_memset(s_battery_run_data.cal_table, 0, table_sums);
        tal_system_memcpy(s_battery_run_data.cal_table, table, table_sums);
        s_battery_run_data.cal_table_len = table_sums;
    } else {
        s_battery_run_data.cal_table = s_battery_table_default;
        s_battery_run_data.cal_table_len = GET_ARRAY_LEN(s_battery_table_default);
    }

    //init adc driver before take sample, deinit after sample completly.
    tal_adc_init(BATTERY_ADC_ID, &(s_battery_run_data.info->adc)); 

    //create the sw timer to sample voltage.
    tal_sw_timer_create(app_battery_event_handler, NULL, &etimer_battery_event_id);

    //heartbeat config
    tal_heartbeat_type_set(HEARTBEAT_BATTERY_VOLTAGE);
    tal_heartbeat_period_set(5 * 60 * 1000);
    tal_heartbeat_start();

    s_battery_run_data.init_flag = TRUE;
}

/**
* @description: call when the device new-join or rejoin.
* @param {type} 
* @return: 
*/
VOID_T app_battery_start(VOID_T)
{
    if (s_battery_run_data.init_flag != TRUE) {
        return;
    }

    s_battery_run_data.first_rpt_real = TRUE;
    tal_sw_timer_stop(etimer_battery_event_id);
    tal_sw_timer_start(etimer_battery_event_id, s_battery_run_data.info->cap_first_delay_time, TAL_TIMER_ONCE);
}

/**
* @description: 
* @param {type} 
* @return: 
*/
VOID_T app_battery_stop(VOID_T)
{
    if (s_battery_run_data.init_flag != TRUE) {
        return;
    }

    tal_sw_timer_stop(etimer_battery_event_id);
    s_battery_run_data.first_rpt_real = TRUE;
}

/**
* @description: 
* @param {type} 
* @return: 
*/
VOID_T app_battery_update_attribute(UINT8_T ep, UINT8_T attr_val)
{
    write_sensor_attribute(ep, CLUSTER_POWER_CONFIG_CLUSTER_ID, ATTR_BATTERY_PERCENTAGE_REMAINING_ATTRIBUTE_ID, &attr_val, ATTR_INT8U_ATTRIBUTE_TYPE);
}

/**
* @description: 
* @param {type} 
* @return: 
*/
VOID_T app_battery_update_and_report_attribute(UINT8_T ep, UINT8_T attr_val)
{
    write_sensor_attribute(ep, CLUSTER_POWER_CONFIG_CLUSTER_ID, ATTR_BATTERY_PERCENTAGE_REMAINING_ATTRIBUTE_ID, &attr_val, ATTR_INT8U_ATTRIBUTE_TYPE);
    report_sensor_attribute(ep, CLUSTER_POWER_CONFIG_CLUSTER_ID, ATTR_BATTERY_PERCENTAGE_REMAINING_ATTRIBUTE_ID, ATTR_INT8U_ATTRIBUTE_TYPE, 500);
}

/**
* @description: 
* @param {type} 
* @return: 
*/
VOID_T app_battery_report_attribute(UINT8_T ep)
{
    report_sensor_attribute(ep, CLUSTER_POWER_CONFIG_CLUSTER_ID, ATTR_BATTERY_PERCENTAGE_REMAINING_ATTRIBUTE_ID, ATTR_INT8U_ATTRIBUTE_TYPE, 500);
}

/**
* @description: 
* @param {type} 
* @return: 
*/
VOID_T app_battery_mf_test_start(VOID_T)
{
    s_battery_run_data.first_rpt_real = TRUE;
    tal_sw_timer_stop(etimer_battery_event_id);
    tal_sw_timer_start(etimer_battery_event_id, 10, TAL_TIMER_ONCE);
}


/**
 * @brief heartbeat report callback
 * @note < this is a weak function you can rewrite this API in app layer >
 * 
 * @param[in]   type: heartbeat type
 *
 * @return none
 */
VOID_T tal_heartbeat_report_callback(TAL_HEARTBEAT_TYPE_E type)
{
    TAL_PR_TRACE("tal_heartbeat_report_callback[%d]\r\n",type);
    //if (type == HEARTBEAT_APP_VERSION) {
      app_battery_report_attribute(0x01);
    //}
}