/*
 * @Author: zhangshu
 * @email: shiliang.zhang@tuya.com
 * @LastEditors: Please set LastEditors
 * @file name: app_dev_register.c
 * @Description: 
 * @Copyright: HANGZHOU TUYA INFORMATION TECHNOLOGY CO.,LTD
 * @Company: http://www.tuya.com
 * @Date: 2022-02-08
 * @LastEditTime: 2022-02-08
 */
#include "app_dev_register.h"
#include "tal_network_mgr.h"

#ifndef GET_ARRAY_LEN
#define GET_ARRAY_LEN(x)    (SIZEOF(x) / SIZEOF(x[0]))
#endif

//power attribute list
#define POWER_ATTR_LIST \
    { 0x0000, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_SINGLETON), 0,   (UINT8_T*)0x0000UL }, /* 10 / Power Configuration / mains voltage*/\
    { 0x0020, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_SINGLETON), 0,   (UINT8_T*)0x00UL  }, /* 11 / Power Configuration / battery voltage*/\
    { 0x0021, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_SINGLETON), 0,   (UINT8_T*)0xC8 } , /* 12 / Power Configuration / battery percentage remaining*/\
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_SINGLETON), 0,   (UINT8_T*)0x0001  }, /* 13 / Power Configuration / cluster revision*/\

//identify attribute list
#define IDENTIFY_ATTR_LIST                                                                                                     \
    {0x0000, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST), 0, {(UINT8_T *)0x0005}},   /* 12 / Identify / identify time*/ \
    {0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, {(UINT8_T *)0x0001}},     /* 13 / Identify / cluster revision*/
//group attribute list
#define GROUP_ATTR_LIST \
    { 0x0000, ATTR_BITMAP8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00 }, /* 12 / Groups / name support*/\
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0002 }, /* 13 / Groups / cluster revision*/
//scene attribute list
#define SCENE_ATTR_LIST \
    { 0x0000, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00 }, /* 15 / Scenes / scene count*/\
    { 0x0001, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00 }, /* 16 / Scenes / current scene*/\
    { 0x0002, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0000 }, /* 16 / Scenes / current group*/\
    { 0x0003, ATTR_BOOLEAN_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00 }, /* 17 / Scenes / scene valid*/\
    { 0x0004, ATTR_BITMAP8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x00 }, /* 18 / Scenes / name support*/\
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0002 }, /* 19 / Scenes / cluster revision*/
//illumination measurement attribute list
#define ILLUM_MEASUREMENT_ATTR_LIST \
    { 0x0000, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0000 }, /* 26 / Illum measurement / cluster revision*/\
    { 0x0001, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0000 }, /* 26 / Illum measurement / cluster revision*/\
    { 0x0002, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T *)0x0000 }, /* 26 / Illum measurement / cluster revision*/\
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, { (UINT8_T *)0x0002 } }, /* 26 / Illum measurement / cluster revision*/
//temperature measurement attribute list
#define TEMPERATURE_ATTR_LIST \
    { 0x0000, ATTR_INT16S_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0000 }, /* 0 / temperature measurement / measured value*/\
    { 0x0001, ATTR_INT16S_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0000 }, /* 1 / temperature measurement / min measured value */\
    { 0x0002, ATTR_INT16S_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0000 }, /* 2 / temperature measurement / max measured value */\
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0002 }, /* 3 / temperature measurement / cluster revision*/
//humidity measurement attribute list
#define HUMIDITY_ATTR_LIST \
    { 0x0000, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0000 }, /* 0 / relative humidity measurement / measured value*/\
    { 0x0001, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0000 }, /* 1 / relative humidity measurement / min measured value */\
    { 0x0002, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0000 }, /* 2 / relative humidity measurement / max measured value */\
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, (UINT8_T*)0x0002 }, /* 3 / relative humidity measurement / cluster revision*/
//ias zone attribute list
#define IAS_ZONE_ATTR_LIST1 \
    { 0x0000, ATTR_ENUM8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST), 0, { (UINT8_T*)0x00 } }, /* 21 / IAS Zone / zone state*/\
    { 0x0001, ATTR_ENUM16_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST), 0, { (UINT8_T*)0x0028 } }, /* 22 / IAS Zone / zone type*/\
    { 0x0002, ATTR_BITMAP16_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, { (UINT8_T*)0x0000 } }, /* 23 / IAS Zone / zone status*/\
    { 0x0010, ATTR_IEEE_ADDRESS_ATTRIBUTE_TYPE, 8, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_WRITABLE), 0, { NULL } }, /* 24 / IAS Zone / IAS CIE address*/\
    { 0x0011, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST), 0, { (UINT8_T*)0xff } }, /* 25 / IAS Zone / Zone ID*/\
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, { (UINT8_T*)0x0001 } }, /* 26 / IAS Zone / cluster revision*/
//tuya private attribute list
#define TUYA_SENSOR_ATTR_LIST \
    { 0xE000, ATTR_ENUM8_ATTRIBUTE_TYPE, 1, (ATTR_MASK_READABLE), 0, { (UINT8_T*)0x02 } }, /* 21 / TUYA sensor / human_motion_state*/\
    { 0xE001, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_WRITABLE), 0, { (UINT8_T*)0x0005 } }, /* 22 / TUYA sensor / presence_delay*/\
    { 0xE002, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_WRITABLE), 0, { (UINT8_T*)0x01C2 } }, /* 23 / TUYA sensor / motion_far_detection*/\
    { 0xE003, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_WRITABLE), 0, { (UINT8_T*)0x01C2 } }, /* 24 / TUYA sensor / motionless_far_detection*/\
    { 0xE004, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_WRITABLE), 0, { (UINT8_T*)0x2D } }, /* 25 / TUYA sensor / motion_sensitivity_value*/\
    { 0xE005, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_WRITABLE), 0, { (UINT8_T*)0x19 } }, /* 25 / TUYA sensor / motionless_sensitivity*/\
    { 0xE006, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST), 0, { (UINT8_T*)0x00 } }, /* 25 / TUYA sensor / motion_amplit_value*/\
    { 0xE007, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST), 0, { (UINT8_T*)0x00 } }, /* 25 / TUYA sensor / Zmotionless_amplit_value*/\
    { 0xE008, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST), 0, { (UINT8_T*)0x00 } }, /* 25 / TUYA sensor / color_temp_value*/\
    { 0xE009, ATTR_INT8U_ATTRIBUTE_TYPE, 1, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_WRITABLE), 0, { (UINT8_T*)0x00 } }, /* 25 / TUYA sensor / led_ind_switch*/\
    { 0xE00A, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, { (UINT8_T*)0x0096 } }, /* 23 / TUYA sensor / current distance*/\
    { 0xE00B, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_TOKEN_FAST|ATTR_MASK_WRITABLE), 0, { (UINT8_T*)0x01C2 } }, /* 25 / TUYA sensor / all far detection*/\
    { 0xE00C, ATTR_BOOLEAN_ATTRIBUTE_TYPE, 1, (ATTR_MASK_WRITABLE), 0, { (UINT8_T*)0x00 } }, /* 23 / TUYA sensor / setting*/\
    { 0xFFFD, ATTR_INT16U_ATTRIBUTE_TYPE, 2, (ATTR_MASK_READABLE), 0, { (UINT8_T*)0x0001 } }, /* 26 / TUYA sensor / cluster revision*/

//power attribute
CONST TAL_ATTR_T power_attr_list[] = {
    POWER_ATTR_LIST
};

// identify attribute
CONST TAL_ATTR_T identify_attr_list[] = {
    IDENTIFY_ATTR_LIST
};
#if 0
// group attribute
CONST TAL_ATTR_T group_attr_list[] = {
    GROUP_ATTR_LIST
};
// scene attribute
CONST TAL_ATTR_T scene_attr_list[] = {
    SCENE_ATTR_LIST
};
// onoff attribute
CONST TAL_ATTR_T temperature_attr_list[] = {
    TEMPERATURE_ATTR_LIST
};
// level attribute
CONST TAL_ATTR_T humidity_attr_list[] = {
    HUMIDITY_ATTR_LIST
};
// illumination attribute
CONST TAL_ATTR_T illum_measurement_attr_list[] = {
    ILLUM_MEASUREMENT_ATTR_LIST
};
#endif
// ias zone attribute
CONST TAL_ATTR_T ias_zone_attr_list[] = {
    IAS_ZONE_ATTR_LIST1
};
// application ias zone server attribute
CONST TAL_ATTR_T tuya_sensor_server_attr_list[] = {
    TUYA_SENSOR_ATTR_LIST
};

#define DEF_CLUSTER_POWER_CLUSTER_ID(a) \
    { CLUSTER_POWER_CONFIG_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },

#define DEF_CLUSTER_IDENTIFY_CLUSTER_ID(a) \
    {CLUSTER_IDENTIFY_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_GROUPS_CLUSTER_ID(a) \
    {CLUSTER_GROUPS_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_SCENES_CLUSTER_ID(a) \
    {CLUSTER_SCENES_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a))},

#define DEF_CLUSTER_TEMP_MEASUREMENT_CLUSTER_ID(a) \
    { CLUSTER_TEMP_MEASUREMENT_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },

#define DEF_CLUSTER_RELATIVE_HUMIDITY_MEASUREMENT_CLUSTER_ID(a) \
    { CLUSTER_RELATIVE_HUMIDITY_MEASUREMENT_CLUSTER_ID , (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },

#define DEF_CLUSTER_ILLUM_MEASUREMENT_CLUSTER_ID(a) \
    { CLUSTER_ILLUM_MEASUREMENT_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },

#define DEF_CLUSTER_IAS_ZONE_CLUSTER_ID(a) \
    { CLUSTER_IAS_ZONE_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },

#define DEF_CLUSTER_TUYA_SENSOR_CLUSTER_ID(a) \
    { CLUSTER_TUYA_SENSOR_CLUSTER_ID, (TAL_ATTR_T *)&((a)[0]), GET_ARRAY_LEN((a)) },

// server cluster list
CONST TAL_CLUSTER_T app_server_cluster_list[] = {
    DEF_CLUSTER_IDENTIFY_CLUSTER_ID(identify_attr_list)
    DEF_CLUSTER_POWER_CLUSTER_ID(power_attr_list)
    DEF_CLUSTER_IAS_ZONE_CLUSTER_ID(ias_zone_attr_list)
    DEF_CLUSTER_TUYA_SENSOR_CLUSTER_ID(tuya_sensor_server_attr_list)
};

#define SERVER_CLUSTER_NUM GET_ARRAY_LEN(app_server_cluster_list)

//endpoint descriptor
TAL_ENDPOINT_T dev_endpoint_desc[] = {
    {1, ZHA_PROFILE_ID, ZG_DEVICE_ID_IAS_ZONE, SERVER_CLUSTER_NUM, (TAL_CLUSTER_T *)&app_server_cluster_list[0], 0, NULL},
};

/**
 * @brief check if the cluster already in registertion list
 * NOTE: only for server clusters attributes
 *
 * @param[in] cluster_id: cluster id
 * @return TRUE if the cluster exists in registertion list or FALSE
 */
BOOL_T is_server_cluster_valid(UINT16_T cluster_id)
{
    for (UINT8_T i = 0; i < SERVER_CLUSTER_NUM; i++) {
        if (app_server_cluster_list[i].cluster_id == cluster_id) {
            return TRUE;
        }
    }
    return FALSE;
}

/**
 * @brief check if the attribute  of cluster already in registertion list
 * NOTE: only for server clusters attributes
 *
 * @param[in] cluster_id: cluster id
 * @param[in] attribute_id: attribute id
 * @return TRUE when in registertion list else FALSE
 */
BOOL_T is_server_attribute_valid(UINT16_T cluster_id, UINT16_T attribute_id)
{
    for (UINT8_T i = 0; i < SERVER_CLUSTER_NUM; i++) {
        if (app_server_cluster_list[i].cluster_id == cluster_id) {
            for (UINT8_T j = 0; j < app_server_cluster_list[i].attr_sums; j++) {
                if (app_server_cluster_list[i].attr_list[j].attr_id == attribute_id) {
                    return TRUE;
                }
            }
        }
    }
    return FALSE;
}

/**
 * @brief register router or sleep device
 * @return none
 */
STATIC VOID_T app_router_node_init(VOID_T)
{
    TAL_ZG_NODE_CFG_T node_config = {
        .node_type = ZG_ROUTER,//zigbee type, router, end device or sleep end device
        .tx_power = DEFAULT_TX_POWER,//set tx power
        .scan_interval = BEACON_SEND_INTERVAL_200MS,//beacon scan interval
        .scan_duration = ZG_SCAN_DURATION_3,//beacon scan interval 153.6 ms
    };
    //config zigbee node
    tal_zg_node_config(&node_config);
}

/**
 * @brief register router or sleep device
 * @return none
 */
STATIC VOID_T app_end_device_node_init(VOID_T)
{
    TAL_ZG_NODE_CFG_T node_config = {
        .node_type = ZG_SLEEPY_END_DEVICE,
        .tx_power = 10,
        .scan_interval = 200,
        .scan_duration = ZG_SCAN_DURATION_3,
        .config.sleep_ed_cfg.poll_config.forever_flag = 0,
        .config.sleep_ed_cfg.poll_config.fast_swicth_parent = 1,
        .config.sleep_ed_cfg.poll_config.interval_ms = 1000,
        .config.sleep_ed_cfg.poll_config.max_failed_times = 5,
        .config.sleep_ed_cfg.poll_config.duration_after_data_ms = 2000,
        .config.sleep_ed_cfg.poll_config.duration_after_join_ms = 30000,
        .config.sleep_ed_cfg.poll_config.duration_after_rejoin_ms = 15000,
        .config.sleep_ed_cfg.rejoin_config.power_on_active = 1,
        .config.sleep_ed_cfg.rejoin_config.send_data_active = 1,
        .config.sleep_ed_cfg.rejoin_config.attempts = 3,
        .config.sleep_ed_cfg.rejoin_config.interval = 1000,
    };
    
    //config zigbee node
    tal_zg_node_config(&node_config);
}

/**
 * @brief config join strategy
 * @return none
 */
STATIC VOID_T dev_joined_config(VOID_T)
{
    TAL_ZG_JOIN_CFG_T join_config = {
        .auto_join_power_on_flag = FALSE,//defalut power on start join or not
        .auto_join_remote_leave_flag = TRUE,//default start join or not when receivew remote leave request
        .join_timeout = ZIGBEE_JOIN_TIMEOUT_MS,//join timeout times
    };
    //config join prameter
    tal_zg_join_config(&join_config);
}

/**
 * @brief register zigbee parameter
 * @return none
 */
VOID_T dev_zigbee_init(VOID_T)
{
    //register zigbee endpoint
    tal_zg_endpoint_register(dev_endpoint_desc, GET_ARRAY_LEN(dev_endpoint_desc));

    //zigbee node configuration
    app_end_device_node_init();

    //zigbee joining network configuration
    dev_joined_config();

    tal_zg_nwk_recovery_disable(TRUE);
}

